<?php

namespace Drupal\monarch_ql\Form;

use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Monarch QL settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * The entity_type.manager service.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity_field.manager service.
   */
  private EntityFieldManagerInterface $entityFieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->entityFieldManager = $container->get('entity_field.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'monarch_ql_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['monarch_ql.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['fields'] = [
      '#type' => 'checkboxes',
      '#options' => [],
      '#title' => $this->t('Fields Allowed in Query or Filter'),
      '#default_value' => $this->config('monarch_ql.settings')->get('fields') ?? [],
    ];

    $definitions = ($this->entityTypeManager->getDefinitions() ?: []);

    ksort($definitions);

    foreach ($definitions as $definition) {
      if (!$definition || !$definition->getBaseTable() || !($definition instanceof ContentEntityTypeInterface)) {
        continue;
      }

      $field_storage_definitions = ($this->entityFieldManager->getFieldStorageDefinitions($definition->id()) ?: []);

      foreach ($field_storage_definitions as $field_storage_definition) {
        $form['fields']['#options'][$definition->id() . '|' . $field_storage_definition->getName()] = $this->t('@entity.<strong>@name</strong>', [
          '@entity' => $definition->id(),
          '@name' => $field_storage_definition->getName(),
        ]);
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('monarch_ql.settings')
      ->set('fields', $form_state->getValue('fields'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
